package cn.iocoder.yudao.module.product.service.coupon;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.module.ecw.dal.dataobject.currency.CurrencyDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.unit.UnitDO;
import cn.iocoder.yudao.module.product.dal.dataobject.coupon.CouponDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.product.dto.CouponAvailableGroupDto;
import cn.iocoder.yudao.module.product.dto.CouponUserAvailableDto;
import cn.iocoder.yudao.module.product.param.ProdConditionParam;
import cn.iocoder.yudao.module.product.vo.coupon.*;

/**
 * 优惠券信息 Service 接口
 *
 * @author 系统管理员
 */
public interface CouponService extends IService<CouponDO> {

    /**
     * 创建优惠券信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCoupon(@Valid CouponCreateReqVO createReqVO);

    /**
     * 更新优惠券信息
     *
     * @param updateReqVO 更新信息
     */
    void updateCoupon(@Valid CouponUpdateReqVO updateReqVO);

    /**
     * 更新优惠过期时间
     *
     * @param couponId   优惠ID
     * @param endTime    有效期时间
     * @param isValidity 是否设置有效期
     */
    void updateCouponEndTime(Long couponId, Date endTime, boolean isValidity);

    /**
     * 删除优惠券信息
     *
     * @param id 编号
     */
    void deleteCoupon(Long id);

    /**
     * 获得优惠券信息
     *
     * @param id 编号
     * @return 优惠券信息
     */
    CouponDO getCoupon(Long id);

    /**
     * 注册赠送优惠劵
     *
     * @param userId 用户ID
     */
    void registerGiveCoupon(Long userId);


    /**
     * 获得用户可用优惠劵列表
     *
     * @param orderType              订单属性： 1 普货  2 重货 3 泡货
     * @param customsType            报关方式: 我司全代：1，自单代报：2，混合报关：3
     * @param isControl              是否控货： 0 否  1 是
     * @param userId                 用户ID
     * @param consignorCustomerId    发货客户ID
     * @param consigneeCustomerId    收货客户ID
     * @param prodConditionParamList 商品条件列表
     * @param lineId                 线路ID
     * @param channelId              渠道ID
     * @return 用户可以用优惠劵列表
     */
    @Deprecated
    CouponUserAvailableDto getCouponUserAvailableDto(Integer orderType, Integer customsType, Integer isControl, Long userId,
                                                     Long consignorCustomerId, Long consigneeCustomerId,
                                                     List<ProdConditionParam> prodConditionParamList,
                                                     Integer transportId, Long lineId, Long channelId);

    /**
     * 获得用户可用商品优惠活动
     *
     * @param orderType                   订单属性： 1 普货  2 重货 3 泡货
     * @param customsType                 报关方式: 我司全代：1，自单代报：2，混合报关：3
     * @param isControl                   是否控货： 0 否  1 是
     * @param userId                      用户ID
     * @param consignorCustomerContactsId 发货客户联系人ID
     * @param consigneeCustomerContactsId 收货客户联系人ID
     * @param lineId                      线路ID
     * @param channelId                   渠道ID
     * @param paramList                   订单商品条件列表
     * @param rmbCurrency                 RMB货币单位
     * @return 商品条件
     */
    void getCouponUserAvailableDto(Integer transportId, Integer orderType, Integer customsType, Integer isControl, Long userId,
                                   Long consignorCustomerContactsId, Long consigneeCustomerContactsId,
                                   Long lineId, Long channelId, List<ProdConditionParam> paramList,
                                   Integer rmbCurrency, Map<Integer, UnitDO> unitDOMap);


    /**
     * 获得用户可用商品优惠活动
     *
     * @param orderType                   订单属性： 1 普货  2 重货 3 泡货
     * @param customsType                 报关方式: 我司全代：1，自单代报：2，混合报关：3
     * @param isControl                   是否控货： 0 否  1 是
     * @param userId                      用户ID
     * @param consignorCustomerContactsId 发货客户联系人ID
     * @param consigneeCustomerContactsId 收货客户联系人ID
     * @param lineId                      线路ID
     * @param channelId                   渠道ID
     * @param param                       商品条件
     * @param unitDOMap                      计量单位
     * @param rmbCurrency                 RMB货币单位
     * @return 商品条件
     */
    CouponAvailableGroupDto getCouponUserAvailableDto(Integer orderType, Integer customsType, Integer isControl, Long userId,
                                                      Long consignorCustomerContactsId, Long consigneeCustomerContactsId,
                                                      Long lineId, Long channelId, ProdConditionParam param, Map<Integer, UnitDO> unitDOMap,
                                                      Integer rmbCurrency);


    /**
     * 获得优惠券信息列表
     *
     * @param ids 编号
     * @return 优惠券信息列表
     */
    List<CouponDO> getCouponList(Collection<Long> ids);

    /**
     * 获得优惠券信息分页
     *
     * @param pageReqVO 分页查询
     * @return 优惠券信息分页
     */
    PageResult<CouponDO> getCouponPage(CouponPageReqVO pageReqVO);

    /**
     * 获得优惠券信息列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 优惠券信息列表
     */
    List<CouponDO> getCouponList(CouponExportReqVO exportReqVO);

    /**
     * 优惠劵详情
     *
     * @param couponId 优惠劵ID
     * @return
     */
    CouponRespVO getInfo(Long couponId);

    List<CouponDO> getCouponSelect(CouponPageReqVO pageVO);


    /**
     * 会员用户的可用优惠劵数量
     *
     * @param userId 会员ID
     * @return 优惠劵数量
     */
    Integer countUserCoupon(Long userId);

    /**
     * 执行修改过期优惠劵的状态
     */
    long couponExpire();

    /**
     * 新增商品可用优惠列表
     *
     * @return 优惠列表
     */
    List<CouponDO> getNewProdCouponList();


    /**
     * 更新优惠券信息
     *
     * @param prodId    商品ID
     * @param couponIds 适用的优惠ID
     */
    void updateProdCoupon(Long prodId, List<Long> couponIds);

    void updateCouponPutonStatus(Long couponId, Integer putonStatus);
}
